///////////////////////////////////////////////////////////////////////////////
//
//            (C) Copyright 2009-2010 OSR Open Systems Resources, Inc.
//            All Rights Reserved
//
//    This work is distributed under the OSR Non-Commercial Software License which is provided
//    at "http://www.osronline.com/page.cfm?name=NonCommLicense" in the hope that it will be
//    enlightening, but WITHOUT ANY WARRANTY; without even the implied warranty of MECHANTABILITY
//    or FITNESS FOR A PARTICULAR PURPOSE.
//
//            OSR Open Systems Resources, Inc.
//            105 Route 101A Suite 19
//            Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
//
//
//
//    MODULE:
//
//        $File: //depot/tools/UMERESOURCES/UmEResourceDll/EResource.h $
//
//    ABSTRACT:
//
//     Defines for the DLL.
//
//    AUTHOR:
//
//        OSR Open Systems Resources, Inc.
// 
//    REVISION:   
//
//        $Revision: #1 $
//
//    NOTES:
//        This code throws exceptions when errors are encountered.   Ensure
//        that you read the code to understand the exceptions thrown.
//
///////////////////////////////////////////////////////////////////////////////
#pragma once
#include "UMERESOURCE.H"
#include <map>

//
// This structure keeps track of the state of the resource.  We need
// to know how the resource was acquired and how many times it was 
// recursively acquired.
//
typedef struct _OSRERESOURCE_STATE
{
    union {
        struct {
            ULONG   State;          // Acquisition State
            ULONG   AcquireCount;   // Count of times acquired
        };
        ULONGLONG   Reserved;
    };
} OSRRESOURCE_STATE, *POSRRESOURCE_STATE;

// OSREResource command target

class OSREResource : public CObject
{
public:
	OSREResource();
	virtual ~OSREResource();

	enum {
		ACQUIRED_NONE,
		ACQUIRED_SHARED,
		ACQUIRED_EXCLUSIVE
	} ACQUIRED_STATE;

	BOOLEAN AcquireResourceExclusive(BOOLEAN Wait);
	BOOLEAN AcquireResourceShared(BOOLEAN Wait);
	BOOLEAN AcquireSharedStarveExclusive(BOOLEAN Wait);
	BOOLEAN IsResourceAcquiredExclusive();
	BOOLEAN IsResourceAcquiredShared();
	VOID ReleaseResource();
	VOID ConvertExclusiveToShared();
	VOID ReleaseResourceForThread(ERESOURCE_THREAD ResourceThreadId);
	ULONG	GetSharedWaiterCount();
	ULONG	GetExclusiveWaiterCount();
	NTSTATUS InitializeResource();
	NTSTATUS DeleteResource();

private:
	mutable CRITICAL_SECTION	m_CriticalSection;

	typedef std::map<DWORD,OSRRESOURCE_STATE> CMapThreadToLockState;
	CMapThreadToLockState m_LockStateMap;

	volatile DWORD	m_SharedCount;
    volatile DWORD  m_StarveExclusiveCount;
	volatile DWORD	m_ExclusiveCount;
	volatile DWORD	m_SharedWaitersCount;
	volatile DWORD	m_ExclusiveWaitersCount;
	HANDLE	m_WaitEventHandle;
	BOOLEAN m_Initialized;
};


